#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import csv
from datetime import date
import os

from django.core.management.base import BaseCommand, CommandError

from ldap_inp import LdapEleve, L
from portailuser.models import Ecole


class Command(BaseCommand):
    """
    Argument (ligne de commande): <fichier> <annee> [raz]
    """

    help = "Inscription frappe ENSAT: <fichier> <annee> [raz]"

    def handle(self, *args, **kwargs):
        if len(args) >= 2:
            fichier = args[0]
            annee = args[1]
            if len(args) == 3 and args[2] == "raz":
                print("RAZ")
                raz_AE_ensat(annee)
            main(fichier)
        else:
            raise CommandError("Vous devez donner un CSV et la promo à reset.")


def raz_AE_ensat(annee):
    """Remet à False le champ inscritAE pour les ENSAT de la promo <annee>"""

    filtre_promo = None
    filtre_promo |= L(promo=str(annee))
    filtre_promo &= L(inscritAE="TRUE")
    print(filtre_promo)

    ecole_ensat = Ecole.objects.get(ident="ensat")
    etus = LdapEleve.objects.filter(L(ecole=ecole_ensat), filtre_promo)
    print(f"Les étudiants réinitialisés: {len(etus)}")

    for etu in etus:
        print(etu)
        etu.inscritAE = False
        etu.save()


def inscr_frappe(user):
    """Incrit une personne à l'AE pour pouvoir utiliser la Frappe ENSAT"""

    etus = LdapEleve.objects.filter(sn=user["nom"], givenName=user["prenom"])
    if len(etus):
        etu = etus[0]
        etu.inscritAE = True
        etu.save()

        return None
    else:
        print(f"{user['prenom']} {user['nom']} : pas dans le LDAP")
        return user


def requete(csv_file):
    """Transform datas in a good query"""
    query = dict(
        {
            "nom": "",
            "prenom": "",
        }
    )
    query_keys = sorted(query.keys())
    date_now = date.today().isoformat()

    with open(csv_file, "r") as f_in:
        freader = csv.reader(f_in, delimiter=",", quotechar='"')
        tmp_filename = f"/tmp/frappe_ensat_{date_now}.csv"
        with open(tmp_filename, "w") as f_out:
            for j, ligne in enumerate(freader):
                if j != 0:
                    # dans le dico
                    eleve = query
                    for i, data in enumerate(query_keys):
                        eleve[data] = ligne[i]

                    # parsing
                    eleve["nom"] = eleve["nom"].title().strip()
                    eleve["prenom"] = eleve["prenom"].title().strip()

                    # inscrit la personne
                    err_user = inscr_frappe(eleve)

                    # si la personne n'est pas dans le LDAP ou autre, on
                    # regénère un CSV
                    if err_user is not None:
                        csv_out = csv.writer(f_out)
                        csv_out.writerow(err_user.values())


def main(csv_file):
    """Main"""
    if os.path.isfile(csv_file):
        print("Frappe !")
        requete(csv_file)
    else:
        raise CommandError("Pas de données")


if __name__ == "__main__":
    main("frappe_ensat_2013.csv")  # promos 2015 & 2016
